#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
#
# Univention Management Console module:
#   Maildisclaimer
#
# Copyright 2017 bitbone AG
#
# http://www.bitbone.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

import sys

sys.path.insert(0, "/usr/share/maildisclaimer")

import copy
import grp
import os
import pwd
import re
import shutil
import subprocess
import urllib
import ldap.filter
import univention.admin.modules

from jinja2 import Template, Environment
from univention.lib.i18n import Translation
from univention.management.console.modules import UMC_OptionTypeError, Base
from univention.management.console.log import MODULE
from univention.management.console.modules.decorators import file_upload
try:
    from univention.management.console.modules.decorators import allow_get_request
except ImportError:
    def allow_get_request(function):
        return function
from univention.management.console.protocol.definitions import *
from bitbone.config import MDConfigReader
from bitbone.utils.license import License
from datetime import datetime
from univention import uldap
import univention.config_registry

from html import HTML

_ = Translation('univention-management-console-module-maildisclaimer').translate

TYPES = ['txt', 'html']


class Instance(Base):
    def init(self):
        ucr = univention.config_registry.ConfigRegistry()
        ucr.load()
        version = ucr['version/version']

        try:
            config = MDConfigReader()
            config.read("/etc/maildisclaimer/server.cfg")
        except IOError, e:
            raise UMC_OptionTypeError('Missing configuration /etc/maildisclaimer/server.cfg. Please contact support@bitbone.de')

        self.template_path = config['disclaimer_dir']+"/templates"
        self.image_path = config['disclaimer_dir']+"/images"
        try:
            self.uid = pwd.getpwnam(config['user']).pw_uid
        except KeyError:
            self.uid = 0

        try:
            self.gid = grp.getgrnam(config['group']).gr_gid
        except KeyError:
            self.gid = 0

        for path in [self.template_path, self.image_path]:
            if not os.path.exists(path):
                os.makedirs(path)
                self._set_permission(path)

        self.lo, self.pos = univention.admin.uldap.getMachineConnection()
        self.posImages = copy.deepcopy(self.pos)
        self.pos.setDn('cn=templates,cn=maildisclaimer,%s' % self.pos.getBase())
        self.posImages.setDn('cn=images,cn=maildisclaimer,%s' % self.pos.getBase())

        univention.admin.modules.update()

        self.md = univention.admin.modules.get('maildisclaimer/templates')
        univention.admin.modules.init(self.lo, self.pos, self.md)

        self.udm_images = univention.admin.modules.get('maildisclaimer/images')
        univention.admin.modules.init(self.lo, self.posImages, self.udm_images)
        self.co = None

        self.commandpath = '/univention-management-console/command/maildisclaimer'

        major, minor = version.split(".")

        if int(major) >= 4 and int(minor) >= 2:
            self.commandpath = '/univention/command/maildisclaimer'

    def _run_it(self, services, action):
        failed = []
        for srv in services:
            if subprocess.call(('/usr/sbin/invoke-rc.d', srv, action)):
                failed.append(srv)
        return failed

    def entries(self):
        files = os.listdir(self.template_path)
        ret = []

        entries = self.md.lookup(self.co, self.lo, ldap.filter.filter_format("cn=*", []))

        for f in entries:
            ret.append({'id': f["name"], 'name': f["name"], 'global': f["global"] == "TRUE" and u"\u2713" or '' }) # u2713 is checkmark

        return ret

    def query(self, request):
        """Searches for entries

        requests.options = {}
          'name' -- search pattern for name (default: '')

        return: [ { 'id' : <unique identifier>, 'name' : <display name> }, ... ]
        """
        MODULE.warn('maildisclaimer.query: options: %s' % str(request.options))
        pattern = request.options.get('name', '')
        result = filter( lambda x: x['name'].find(pattern) >= 0, self.entries() )
        MODULE.warn('maildisclaimer.query: results: %s' % str(result))
        self.finished(request.id, result)

    def get(self, request):
        """Returns the objects for the given IDs

        requests.options = [ <ID>, ... ]

        return: [ { 'id' : <unique identifier>, 'content_html' : <content html>, 'content_txt' : <content txt> }, ... ]
        """
        MODULE.warn('maildisclaimer.get: options: %s' % str(request.options))
        ids = request.options
        result = []
        if isinstance(ids, (list, tuple)):
            ids = set(ids)
            result = filter(lambda x: x['id'] in ids, self.entries())
        else:
            MODULE.warn('maildisclaimer.get: wrong parameter, expected list of strings, but got: %s' % str(ids))
            raise UMC_OptionTypeError('Expected list of strings, but got: %s' % str(ids))

        entries = self.md.lookup(self.co, self.lo, ldap.filter.filter_format("cn=%s", [result[0]["id"]]))

        if len(entries) == 1:
            result[0]["content_txt"] = entries[0]["text"]
            result[0]["content_html"] = entries[0]["html"]
        else:
            raise UMC_OptionTypeError('Too many entries found.')

        MODULE.warn('maildisclaimer.get: results: %s' % str(result))
        self.finished(request.id, result)

    def _set_permission(self, filename):
        os.chown(filename, self.uid, self.gid)

    def add(self, request):
        MODULE.warn("maildisclaimer.add options %s" % repr(request.options))

        name = request.options.get("name", "")

        if name == "global":
            request.status = MODULE_ERR
            self.finished(request.id, { "success": False }, _('Name "global" is reserved!'))
            return

        entries = self.md.lookup(self.co, self.lo, ldap.filter.filter_format("cn=%s", [name]))

        if len(entries) == 0:
            # no entry found, so we create one
            md_obj = self.md.object(self.co, self.lo, self.pos)
            operation = "create"
        elif len(entries) == 1:
            # found one entry, so modify it
            md_obj = entries[0]
            operation = "modify"
        else:
            raise UMC_OptionTypeError('Too many entries found.')

        md_obj.open()

        # only clear global template setting if it is set and differs from the
        # existing setting
        if "global" in md_obj \
                and request.options.get("global", False) \
                and (md_obj["global"] == "TRUE") != request.options.get("global", False):
            self._clear_global_template()

        md_obj['name'] = name
        md_obj['html'] = request.options.get("content_html", "")
        md_obj['text'] = request.options.get("content_txt", "")
        md_obj['global'] = request.options.get("global", False) and "TRUE" or "FALSE"

        if operation == "create":
            md_obj.create()
        elif operation == "modify":
            md_obj.modify()

        self.finished(request.id, True)

    def _clear_global_template(self):
        entries = self.md.lookup(self.co, self.lo, ldap.filter.filter_format("isGlobal=TRUE", []))

        if len(entries) > 0:
            for md_obj in entries:
                MODULE.warn("removing global flag from %s" % md_obj["name"])
                md_obj['global'] = "FALSE"
                md_obj.modify()

        return

    def get_template_used_by(self, request):
        MODULE.warn("maildisclaimer.get_used_template options %s" % repr(request.options))

        names = request.options

        ret = {"users": [], "domains": [], "groups": []}

        if names:
            entries = self._get_ldap_entries(request.options)

            for entry in entries:
                objectClass = entry[1]["objectClass"]

                if 'posixAccount' in objectClass:
                    ret["users"].extend(entry[1]["uid"])

                if 'univentionMailDomainname' in objectClass:
                    ret["domains"].extend(entry[1]["cn"])

                if 'posixGroup' in objectClass:
                    ret["groups"].extend(entry[1]["cn"])

        MODULE.warn("maildisclaimer.get_used_template returned %s" % repr(ret))

        self.finished(request.id, ret)

    def _get_ldap_entries(self, templates):
        templates = map(lambda x: "(maildisclaimerTemplate=%s)" %
                ldap.filter.escape_filter_chars(x), templates)
        connection = uldap.getMachineConnection()

        MODULE.warn("maildisclaimer.get_used_template filter %s" % "".join(templates))

        entries = connection.search(filter="(|%s)" % "".join(templates))

        return entries

    def _clear_ldap_entries(self, templates):
        entries = self._get_ldap_entries(templates)
        connection = uldap.getMachineConnection()
        for entry in entries:
            dn = entry[0]
            mod = [("maildisclaimerTemplate", True, None)]
            connection.modify(dn, mod)

    def remove(self, request):
        MODULE.warn("maildisclaimer.remove options %s" % repr(request.options))

        remove = []
        for item in request.options:
            entries = self.md.lookup(self.co, self.lo, ldap.filter.filter_format("cn=%s", [item]))
            if len(entries) == 1:
                md_obj = entries[0]
                md_obj.open()
                md_obj.remove()
            elif len(entries) == 0:
                raise UMC_OptionTypeError('Entry not found.')
            else:
                raise UMC_OptionTypeError('Too many entries found.')
            remove.append(ldap.filter.escape_filter_chars(item))

        self._clear_ldap_entries(request.options)

        self.finished(request.id, True)

    def _imageStoredSuccess(self, callback, name, source):

        template = Template(HTML.imageStored)

        return template.render(callback=callback, img_src=self._imageUrl(name), _=_).encode("utf-8")

    @file_upload
    def uploadimage(self, request):
        MODULE.warn("maildisclaimer.uploadimage options %s" % repr(request.options))

        tmpfile = request.options[0]["tmpfile"]
        fd = open(tmpfile, "rb")

        image_obj = self.udm_images.object(self.co, self.lo, self.posImages)
        filename = self._get_safe_filename(request.options[0]["filename"])
        filename = filename.encode("raw_unicode_escape")
        image_obj["name"] = filename
        image_obj["content"] = fd.read()
        image_obj.create()

        fd.close()

        self.finished(request.id, self._imageStoredSuccess("0", filename, "source"), mimetype="text/html")

    def _imageUrl(self, filename):
        return "%s/getimage?name=%s" % (self.commandpath, urllib.quote(filename.encode("utf-8")))

    def _createGallery(self):

        images = []

        try:
            entries = self.udm_images.lookup(self.co, self.lo, ldap.filter.filter_format("cn=*", []))
            for f in entries:
                image_link = "javascript:window.opener.CKEDITOR.tools.callFunction('0', '%s');window.close();" % self._imageUrl(f["name"])
                images.append({
                    "src": self._imageUrl(f["name"]),
                    "name": f["name"],
                    "href": "javascript:%s" % urllib.quote(image_link)
                    })
        except OSError:
            pass

        template = Template(HTML.gallery)

        return template.render(images=images, commandpath=self.commandpath, css=HTML.galleryCss, _=_).encode("utf-8")

    @allow_get_request
    def getimage(self, request):
        MODULE.warn("maildisclaimer.getimage options %s" % repr(request.options))

        files = request.options.get("name", "")

        if type(files) is str and files != "":
            files = [files]

        if len(files) > 0:
            if request.options.get("action", "") == _("Delete"):
                for filename in files:
                    filename = self._get_safe_filename(filename)
                    entries = self.udm_images.lookup(self.co, self.lo, ldap.filter.filter_format("cn=%s", [filename]))

                    if len(entries) == 1:
                        MODULE.warn("delete file %s" % filename)

                        md_obj = entries[0]
                        md_obj.open()
                        md_obj.remove()

                self.finished(request.id, self._createGallery(), mimetype="text/html")
            else:
                filename = files[0]
                filename = self._get_safe_filename(filename)

                entries = self.udm_images.lookup(self.co, self.lo, ldap.filter.filter_format("cn=%s", [filename]))
                MODULE.warn("output file %s" % filename)

                if len(entries) == 1:
                    self.finished(request.id, entries[0]["content"], mimetype="image/jpg")
                else:
                    raise UMC_OptionTypeError('Invalid request')

        else:
            MODULE.warn("output gallery")
            self.finished(request.id, self._createGallery(), mimetype="text/html")

    def get_license(self, request):
        MODULE.warn("maildisclaimer.get_license options %s" % repr(request.options))
        license = License()
        key = license.read_key()
        report = license.report()
        ret = ""

        if len(key):
            ret += _("Name: %s\n") % key["Customer"]
            ret += _("Type: %s\n") % key["Type"]
            ret += _("Users: %s\n") % key["Users"]
            ret += _("Version: %s\n") % key["Version"]
            ret += _("Install date: %s\n") % datetime.fromtimestamp(license.get_install_time()).strftime("%Y-%m-%d")
            if key["Type"] == "trial":
                ret += _("Days: %s\n") % key["Days"]

        if report:
            ret += report

        self.finished(request.id, ret)

    def add_license(self, request):
        MODULE.warn("maildisclaimer.add_license options %s" % repr(request.options))

        content = request.options.get("upload").decode("base64")
        license = License()
        new_license = license.replace_license(content=content)

        self._run_it(["maildisclaimer"], "restart")

        self.finished(request.id, new_license)

    def _get_safe_filename(self, filename):
        return re.sub(r"(\/|\\|\0)", "", filename)

